@@ -10,10 +10,10 @@ import tomlkit
 from pip._vendor.pkg_resources import safe_name
 from resolvelib.resolvers import ResolutionImpossible, ResolutionTooDeep
 
+from pdm import termui
 from pdm.cli.utils import (
     check_project_file,
     find_importable_files,
-    find_python_in_path,
     format_lockfile,
     format_resolution_impossible,
     save_version_specifiers,
@@ -23,7 +23,6 @@ from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.formats import FORMATS
 from pdm.formats.base import array_of_inline_tables, make_array, make_inline_table
 from pdm.installers.installers import format_dist
-from pdm.iostream import LOCK, stream
 from pdm.models.builders import EnvBuilder
 from pdm.models.candidates import Candidate
 from pdm.models.in_process import get_python_version
@@ -45,7 +44,7 @@ def do_lock(
     """Performs the locking process and update lockfile.
 
     :param project: the project instance
-    :param strategy: update stratege: reuse/eager/all
+    :param strategy: update strategy: reuse/eager/all
     :param tracked_names: required when using eager strategy
     :param requirements: An optional dictionary of requirements, read from pyproject
         if not given.
@@ -58,12 +57,11 @@ def do_lock(
             r for deps in project.all_dependencies.values() for r in deps.values()
         ]
     resolve_max_rounds = int(project.config["strategy.resolve_max_rounds"])
-    with stream.logging("lock"):
+    ui = project.core.ui
+    with ui.logging("lock"):
         # The context managers are nested to ensure the spinner is stopped before
         # any message is thrown to the output.
-        with stream.open_spinner(
-            title="Resolving dependencies", spinner="dots"
-        ) as spin:
+        with ui.open_spinner(title="Resolving dependencies", spinner="dots") as spin:
             reporter = project.get_reporter(requirements, tracked_names, spin)
             resolver = project.core.resolver_class(provider, reporter)
             try:
@@ -74,22 +72,22 @@ def do_lock(
                     resolve_max_rounds,
                 )
             except ResolutionTooDeep:
-                spin.fail(f"{LOCK} Lock failed")
-                stream.echo(
+                spin.fail(f"{termui.Emoji.LOCK} Lock failed")
+                ui.echo(
                     "The dependency resolution exceeds the maximum loop depth of "
                     f"{resolve_max_rounds}, there may be some circular dependencies "
                     "in your project. Try to solve them or increase the "
-                    f"{stream.green('`strategy.resolve_max_rounds`')} config.",
+                    f"{termui.green('`strategy.resolve_max_rounds`')} config.",
                     err=True,
                 )
                 raise
             except ResolutionImpossible as err:
-                spin.fail(f"{LOCK} Lock failed")
-                stream.echo(format_resolution_impossible(err), err=True)
+                spin.fail(f"{termui.Emoji.LOCK} Lock failed")
+                ui.echo(format_resolution_impossible(err), err=True)
                 raise
             else:
                 data = format_lockfile(mapping, dependencies, summaries)
-                spin.succeed(f"{LOCK} Lock successful")
+                spin.succeed(f"{termui.Emoji.LOCK} Lock successful")
     project.write_lockfile(data)
 
     return mapping
@@ -107,7 +105,7 @@ def do_sync(
 
     :param project: The project instance.
     :param sections: A tuple of optional sections to be synced.
-    :param dev: whether to include dev-dependecies.
+    :param dev: whether to include dev-dependencies.
     :param default: whether to include default dependencies.
     :param dry_run: Print actions without actually running them.
     :param clean: whether to remove unneeded packages.
@@ -119,7 +117,7 @@ def do_sync(
     for section in sections:
         if section not in list(project.iter_sections()):
             raise PdmUsageError(
-                f"Section {stream.green(repr(section))} doesn't exist "
+                f"Section {termui.green(repr(section))} doesn't exist "
                 "in the pyproject.toml"
             )
         candidates.update(project.get_locked_candidates(section))
@@ -144,7 +142,7 @@ def do_add(
     """Add packages and install
 
     :param project: the project instance
-    :param dev: add to dev dependencies seciton
+    :param dev: add to dev dependencies section
     :param section: specify section to be add to
     :param sync: whether to install added packages
     :param save: save strategy
@@ -165,9 +163,9 @@ def do_add(
         r.from_section = section
         tracked_names.add(key)
         requirements[key] = r
-    stream.echo(
+    project.core.ui.echo(
         f"Adding packages to {section} dependencies: "
-        + ", ".join(stream.green(key or "", bold=True) for key in requirements)
+        + ", ".join(termui.green(key or "", bold=True) for key in requirements)
     )
     all_dependencies = project.all_dependencies
     all_dependencies.setdefault(section, {}).update(requirements)
@@ -205,7 +203,7 @@ def do_update(
 
     :param project: The project instance
     :param dev: whether to update dev dependencies
-    :param sections: update speicified sections
+    :param sections: update specified sections
     :param default: update default
     :param strategy: update strategy (reuse/eager)
     :param save: save strategy (compatible/exact/wildcard)
@@ -216,7 +214,7 @@ def do_update(
     check_project_file(project)
     if len(packages) > 0 and (len(sections) > 1 or not default):
         raise PdmUsageError(
-            "packages argument can't be used together with multple -s or --no-default."
+            "packages argument can't be used together with multiple -s or --no-default."
         )
     if not packages:
         if unconstrained:
@@ -244,16 +242,16 @@ def do_update(
         if not matched_name:
             raise ProjectError(
                 "{} does not exist in {} dependencies.".format(
-                    stream.green(name, bold=True), section
+                    termui.green(name, bold=True), section
                 )
             )
         if unconstrained:
             dependencies[matched_name].specifier = get_specifier("")
         tracked_names.add(matched_name)
         updated_deps[matched_name] = dependencies[matched_name]
-    stream.echo(
+    project.core.ui.echo(
         "Updating packages: {}.".format(
-            ", ".join(stream.green(v, bold=True) for v in tracked_names)
+            ", ".join(termui.green(v, bold=True) for v in tracked_names)
         )
     )
     reqs = [r for deps in all_dependencies.values() for r in deps.values()]
@@ -291,9 +289,9 @@ def do_remove(
         raise ProjectError(f"No {section} dependencies given in pyproject.toml.")
 
     deps = project.get_pyproject_dependencies(section)
-    stream.echo(
+    project.core.ui.echo(
         f"Removing packages from {section} dependencies: "
-        + ", ".join(str(stream.green(name, bold=True)) for name in packages)
+        + ", ".join(str(termui.green(name, bold=True)) for name in packages)
     )
     for name in packages:
         req = parse_requirement(name)
@@ -303,7 +301,7 @@ def do_remove(
         if not matched_indexes:
             raise ProjectError(
                 "{} does not exist in {} dependencies.".format(
-                    stream.green(name, bold=True), section
+                    termui.green(name, bold=True), section
                 )
             )
         for i in matched_indexes:
@@ -320,7 +318,7 @@ def do_list(project: Project, graph: bool = False, reverse: bool = False) -> Non
 
     :param project: the project instance.
     :param graph: whether to display a graph.
-    :param reverse: wheter to display reverse graph.
+    :param reverse: whether to display reverse graph.
     """
     from pdm.cli.utils import (
         build_dependency_graph,
@@ -339,13 +337,13 @@ def do_list(project: Project, graph: bool = False, reverse: bool = False) -> Non
             graph = format_reverse_dependency_graph(project, dep_graph)
         else:
             graph = format_dependency_graph(project, dep_graph)
-        stream.echo(graph)
+        project.core.ui.echo(graph)
     else:
         rows = [
-            (stream.green(k, bold=True), format_dist(v))
+            (termui.green(k, bold=True), format_dist(v))
             for k, v in sorted(working_set.items())
         ]
-        stream.display_columns(rows, ["Package", "Version"])
+        project.core.ui.display_columns(rows, ["Package", "Version"])
 
 
 def do_build(
@@ -360,23 +358,23 @@ def do_build(
         raise ProjectError("Not allowed to build based on the global project.")
     check_project_file(project)
     if not wheel and not sdist:
-        stream.echo("All artifacts are disabled, nothing to do.", err=True)
+        project.core.ui.echo("All artifacts are disabled, nothing to do.", err=True)
         return
     if not os.path.isabs(dest):
         dest = project.root.joinpath(dest).as_posix()
     if clean:
         shutil.rmtree(dest, ignore_errors=True)
-    with stream.logging("build"), EnvBuilder(
+    with project.core.ui.logging("build"), EnvBuilder(
         project.root, project.environment
     ) as builder:
         if sdist:
-            stream.echo("Building sdist...")
+            project.core.ui.echo("Building sdist...")
             loc = builder.build_sdist(dest)
-            stream.echo(f"Built sdist at {loc}")
+            project.core.ui.echo(f"Built sdist at {loc}")
         if wheel:
-            stream.echo("Building wheel...")
+            project.core.ui.echo("Building wheel...")
             loc = builder.build_wheel(dest)
-            stream.echo(f"Built wheel at {loc}")
+            project.core.ui.echo(f"Built wheel at {loc}")
 
 
 def do_init(
@@ -418,59 +416,50 @@ def do_use(project: Project, python: str, first: bool = False) -> None:
     """Use the specified python version and save in project config.
     The python can be a version string or interpreter path.
     """
-    import pythonfinder
+
+    def version_matcher(py_version):
+        return project.python_requires.contains(str(py_version.version))
 
     python = python.strip()
-    if python and not all(c.isdigit() for c in python.split(".")):
-        if Path(python).exists():
-            python_path = find_python_in_path(python)
-        else:
-            python_path = shutil.which(python)
-        if not python_path:
-            raise NoPythonVersion(f"{python} is not a valid Python.")
-        python_version, is_64bit = get_python_version(python_path, True)
-    else:
-        finder = pythonfinder.Finder()
-        pythons = []
-        args = [int(v) for v in python.split(".") if v != ""]
-        for i, entry in enumerate(finder.find_all_python_versions(*args)):
-            python_version, is_64bit = get_python_version(entry.path.as_posix(), True)
-            pythons.append((entry.path.as_posix(), python_version, is_64bit))
-        if not pythons:
-            raise NoPythonVersion(f"Python {python} is not available on the system.")
 
-        if not first and len(pythons) > 1:
-            for i, (path, python_version, is_64bit) in enumerate(pythons):
-                stream.echo(
-                    f"{i}. {stream.green(path)} "
-                    f"({get_python_version_string(python_version, is_64bit)})"
-                )
-            selection = click.prompt(
-                "Please select:",
-                type=click.Choice([str(i) for i in range(len(pythons))]),
-                default="0",
-                show_choices=False,
-            )
-        else:
-            selection = 0
-        python_path, python_version, is_64bit = pythons[int(selection)]
-
-    if not project.python_requires.contains(python_version):
+    found_interpreters = list(
+        dict.fromkeys(filter(version_matcher, project.find_interpreters(python)))
+    )
+    if not found_interpreters:
         raise NoPythonVersion(
-            "The target Python version {} doesn't satisfy "
-            "the Python requirement: {}".format(python_version, project.python_requires)
+            f"Python interpreter {python} is not found on the system."
         )
-    stream.echo(
+    if first or len(found_interpreters) == 1:
+        selected_python = found_interpreters[0]
+    else:
+        for i, py_version in enumerate(found_interpreters):
+            python_version = str(py_version.version)
+            is_64bit = py_version.get_architecture() == "64bit"
+            version_string = get_python_version_string(python_version, is_64bit)
+            project.core.ui.echo(
+                f"{i}. {termui.green(py_version.executable)} ({version_string})"
+            )
+        selection = click.prompt(
+            "Please select:",
+            type=click.Choice([str(i) for i in range(len(found_interpreters))]),
+            default="0",
+            show_choices=False,
+        )
+        selected_python = found_interpreters[int(selection)]
+
+    old_path = project.config.get("python.path")
+    new_path = selected_python.executable
+    python_version = str(selected_python.version)
+    is_64bit = selected_python.get_architecture() == "64bit"
+    project.core.ui.echo(
         "Using Python interpreter: {} ({})".format(
-            stream.green(python_path),
+            termui.green(str(new_path)),
             get_python_version_string(python_version, is_64bit),
         )
     )
-    old_path = project.config.get("python.path")
-    new_path = python_path
-    project.project_config["python.path"] = Path(new_path).as_posix()
+    project.python_executable = new_path
     if old_path and Path(old_path) != Path(new_path) and not project.is_global:
-        stream.echo(stream.cyan("Updating executable scripts..."))
+        project.core.ui.echo(termui.cyan("Updating executable scripts..."))
         project.environment.update_shebangs(new_path)
 
 
@@ -486,23 +475,25 @@ def do_info(
     python_version, is_64bit = get_python_version(python_path, True)
     if not python and not show_project and not env:
         rows = [
-            (stream.cyan("PDM version:", bold=True), project.core.version),
+            (termui.cyan("PDM version:", bold=True), project.core.version),
             (
-                stream.cyan("Python Interpreter:", bold=True),
+                termui.cyan("Python Interpreter:", bold=True),
                 python_path
                 + f" ({get_python_version_string(python_version, is_64bit)})",
             ),
-            (stream.cyan("Project Root:", bold=True), project.root.as_posix()),
+            (termui.cyan("Project Root:", bold=True), project.root.as_posix()),
         ]
-        stream.display_columns(rows)
+        project.core.ui.display_columns(rows)
         return
 
     if python:
-        stream.echo(python_path)
+        project.core.ui.echo(python_path)
     if show_project:
-        stream.echo(project.root.as_posix())
+        project.core.ui.echo(project.root.as_posix())
     if env:
-        stream.echo(json.dumps(project.environment.marker_environment, indent=2))
+        project.core.ui.echo(
+            json.dumps(project.environment.marker_environment, indent=2)
+        )
 
 
 def do_import(
@@ -558,15 +549,15 @@ def ask_for_import(project: Project) -> None:
     importable_files = list(find_importable_files(project))
     if not importable_files:
         return
-    stream.echo(
-        stream.cyan("Found following files from other formats that you may import:")
+    project.core.ui.echo(
+        termui.cyan("Found following files from other formats that you may import:")
     )
     for i, (key, filepath) in enumerate(importable_files):
-        stream.echo(f"{i}. {stream.green(filepath.as_posix())} ({key})")
-    stream.echo(
+        project.core.ui.echo(f"{i}. {termui.green(filepath.as_posix())} ({key})")
+    project.core.ui.echo(
         "{}. {}".format(
             len(importable_files),
-            stream.yellow("don't do anything, I will import later."),
+            termui.yellow("don't do anything, I will import later."),
         )
     )
     choice = click.prompt(
@@ -577,15 +568,29 @@ def ask_for_import(project: Project) -> None:
     if int(choice) == len(importable_files):
         return
     key, filepath = importable_files[int(choice)]
-    do_import(project, filepath, key)
+    do_import(project, filepath, key, Namespace())
 
 
-def print_pep582_command(shell: str = "AUTO"):
+def print_pep582_command(ui: termui.UI, shell: str = "AUTO"):
     """Print the export PYTHONPATH line to be evaluated by the shell."""
     import shellingham
 
     if os.name == "nt":
-        set_env_in_reg("PYTHONPATH", PEP582_PATH)
+        try:
+            set_env_in_reg("PYTHONPATH", PEP582_PATH)
+        except PermissionError:
+            ui.echo(
+                termui.red(
+                    "Permission denied, please run the terminal as administrator."
+                ),
+                err=True,
+            )
+        ui.echo(
+            termui.green(
+                "The environment variable has been saved, "
+                "please restart the session to take effect."
+            )
+        )
         return
     lib_path = PEP582_PATH.replace("'", "\\'")
     if shell == "AUTO":
@@ -602,7 +607,7 @@ def print_pep582_command(shell: str = "AUTO"):
             f"Unsupported shell: {shell}, please specify another shell "
             "via `--pep582 <SHELL>`"
         )
-    stream.echo(result)
+    ui.echo(result)
 
 
 def migrate_pyproject(project: Project):
@@ -615,14 +620,14 @@ def migrate_pyproject(project: Project):
     ):
         return
 
-    stream.echo(
-        stream.yellow("Legacy [tool.pdm] metadata detected, migrating to PEP 621..."),
+    project.core.ui.echo(
+        termui.yellow("Legacy [tool.pdm] metadata detected, migrating to PEP 621..."),
         err=True,
     )
     do_import(project, project.pyproject_file, "legacy")
-    stream.echo(
-        stream.green("pyproject.toml")
-        + stream.yellow(
+    project.core.ui.echo(
+        termui.green("pyproject.toml")
+        + termui.yellow(
             " has been migrated to PEP 621 successfully. "
             "Now you can safely delete the legacy metadata under [tool.pdm] table."
         ),
