@@ -4,7 +4,6 @@ from pathlib import Path
 from pdm.cli.commands.base import BaseCommand
 from pdm.cli.options import sections_group
 from pdm.formats import FORMATS
-from pdm.iostream import stream
 from pdm.project import Project
 
 
@@ -64,4 +63,4 @@ class Command(BaseCommand):
         if options.output:
             Path(options.output).write_text(content)
         else:
-            stream.echo(content)
+            project.core.ui.echo(content)
