@@ -12,6 +12,16 @@ class Command(BaseCommand):
     name = "import"
 
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
+        parser.add_argument(
+            "-d",
+            "--dev",
+            default=False,
+            action="store_true",
+            help="import packages into dev dependencies",
+        )
+        parser.add_argument(
+            "-s", "--section", help="Specify target section to import into"
+        )
         parser.add_argument(
             "-f",
             "--format",
@@ -22,4 +32,4 @@ class Command(BaseCommand):
         parser.set_defaults(project=self.project_class("."))
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
-        do_import(project, options.filename, options.format)
+        do_import(project, options.filename, options.format, options)
