@@ -2,15 +2,12 @@ import argparse
 
 from pdm.cli import actions
 from pdm.cli.commands.base import BaseCommand
-from pdm.cli.options import global_option
 from pdm.project import Project
 
 
 class Command(BaseCommand):
     """Show the project information"""
 
-    arguments = [global_option]
-
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         parser.add_argument(
             "--python", action="store_true", help="Show the interpreter path"
