@@ -11,21 +11,17 @@ import click
 from pip._vendor import pkg_resources
 from resolvelib import Resolver
 
+from pdm import termui
 from pdm.cli.actions import migrate_pyproject, print_pep582_command
 from pdm.cli.commands.base import BaseCommand
 from pdm.cli.options import ignore_python_option, pep582_option, verbose_option
 from pdm.cli.utils import PdmFormatter, PdmParser
+from pdm.exceptions import PdmUsageError
 from pdm.installers import Synchronizer
-from pdm.iostream import stream
 from pdm.models.repositories import PyPIRepository
 from pdm.project import Project
 from pdm.project.config import Config, ConfigItem
 
-try:
-    import importlib.metadata as importlib_metadata
-except ModuleNotFoundError:
-    import importlib_metadata
-
 COMMANDS_MODULE_PATH = importlib.import_module("pdm.cli.commands").__path__
 
 
@@ -33,6 +29,10 @@ class Core:
     """A high level object that manages all classes and configurations"""
 
     def __init__(self):
+        try:
+            import importlib.metadata as importlib_metadata
+        except ModuleNotFoundError:
+            import importlib_metadata
         self.version = importlib_metadata.version(__name__.split(".")[0])
 
         self.project_class = Project
@@ -40,6 +40,7 @@ class Core:
         self.resolver_class = Resolver
         self.synchronizer_class = Synchronizer
 
+        self.ui = termui.UI()
         self.parser = None
         self.subparsers = None
 
@@ -67,7 +68,7 @@ class Core:
         for _, name, _ in pkgutil.iter_modules(COMMANDS_MODULE_PATH):
             module = importlib.import_module(f"pdm.cli.commands.{name}", __name__)
             try:
-                klass = module.Command  # type: Type[BaseCommand]
+                klass = module.Command
             except AttributeError:
                 continue
             self.register_command(klass, klass.name or name)
@@ -106,12 +107,12 @@ class Core:
         self.load_plugins()
 
         options = self.parser.parse_args(args or None)
-        stream.set_verbosity(options.verbose)
+        self.ui.set_verbosity(options.verbose)
         if options.ignore_python:
             os.environ["PDM_IGNORE_SAVED_PYTHON"] = "1"
 
         if options.pep582:
-            print_pep582_command(options.pep582)
+            print_pep582_command(self.ui, options.pep582)
             sys.exit(0)
 
         self.ensure_project(options, obj)
@@ -127,12 +128,14 @@ class Core:
                     f(options.project, options)
             except Exception:
                 etype, err, traceback = sys.exc_info()
-                if stream.verbosity > stream.NORMAL:
+                should_show_tb = not isinstance(err, PdmUsageError)
+                if self.ui.verbosity > termui.NORMAL and should_show_tb:
                     raise err.with_traceback(traceback)
-                stream.echo(
-                    f"{stream.red('[' + etype.__name__ + ']')}: {err}", err=True
+                self.ui.echo(
+                    f"{termui.red('[' + etype.__name__ + ']')}: {err}", err=True
                 )
-                stream.echo(stream.yellow("Add '-v' to see the detailed traceback"))
+                if should_show_tb:
+                    self.ui.echo("Add '-v' to see the detailed traceback", fg="yellow")
                 sys.exit(1)
 
     def register_command(
@@ -159,9 +162,10 @@ class Core:
             ...
 
         """
-        for plugin in pkg_resources.iter_entry_points("pdm.plugin"):
+        for plugin in pkg_resources.iter_entry_points("pdm"):
             plugin.load()(self)
 
 
-# the main object, which can also act as a callable
-main = Core()
+def main(args=None):
+    """The CLI entry function"""
+    return Core().main(args)
