@@ -9,8 +9,8 @@ from distlib.database import EggInfoDistribution
 from distlib.wheel import Wheel
 from pip._vendor.pkg_resources import safe_extra
 
+from pdm import termui
 from pdm.exceptions import BuildError, ExtrasError, RequirementError
-from pdm.iostream import stream
 from pdm.models import pip_shims
 from pdm.models.markers import Marker
 from pdm.models.readers import SetupReader
@@ -89,13 +89,12 @@ class Candidate:
 
     def __init__(
         self,
-        req,  # type: Requirement
-        environment,  # type: Environment
-        name=None,  # type: Optional[str]
-        version=None,  # type: Optional[str]
-        link=None,  # type: pip_shims.Link
+        req: Requirement,
+        environment: Environment,
+        name: Optional[str] = None,
+        version: Optional[str] = None,
+        link: Optional[pip_shims.Link] = None,
     ):
-        # type: (...) -> None
         """
         :param req: the requirement that produces this candidate.
         :param environment: the bound environment instance.
@@ -110,7 +109,7 @@ class Candidate:
         if link is None and self.req:
             link = self.ireq.link
         self.link = link
-        self.hashes = None  # type: Optional[Dict[str, str]]
+        self.hashes: Optional[Dict[str, str]] = None
         self.marker = None
         self.sections = []
         self._requires_python = None
@@ -137,6 +136,8 @@ class Candidate:
     def revision(self) -> str:
         if not self.req.is_vcs:
             raise AttributeError("Non-VCS candidate doesn't have revision attribute")
+        if self.req.revision:
+            return self.req.revision
         return vcs.get_backend(self.req.vcs).get_revision(self.ireq.source_dir)
 
     def get_metadata(self, allow_all_wheels: bool = True) -> Optional[Metadata]:
@@ -151,7 +152,7 @@ class Candidate:
         try:
             built = self.environment.build(ireq, self.hashes, allow_all_wheels)
         except BuildError:
-            stream.logger.warn("Failed to build package, try parsing project files.")
+            termui.logger.warn("Failed to build package, try parsing project files.")
             meta_dict = SetupReader.read_from_directory(ireq.unpacked_source_directory)
             meta_dict["requires_python"] = meta_dict.pop("python_requires", None)
             self.metadata = Namespace(**meta_dict)
@@ -182,11 +183,10 @@ class Candidate:
     @classmethod
     def from_installation_candidate(
         cls,
-        candidate,  # type: pip_shims.InstallationCandidate
-        req,  # type: Requirement
-        environment,  # type: Environment
-    ):
-        # type: (...) -> Candidate
+        candidate: pip_shims.InstallationCandidate,
+        req: Requirement,
+        environment: Environment,
+    ) -> Candidate:
         """Build a candidate from pip's InstallationCandidate."""
         inst = cls(
             req,
@@ -205,10 +205,16 @@ class Candidate:
             if not metadata:
                 return []
             return get_requirements_from_dist(self.ireq.get_dist(), extras)
-        elif getattr(metadata, "install_requires", None):
-            requires = metadata.install_requires
+        elif hasattr(metadata, "install_requires"):
+            requires = metadata.install_requires or []
+            extras_not_found = set()
             for extra in extras:
-                requires.extend(metadata.extras_require.get(extra, []))
+                try:
+                    requires.extend((metadata.extras_require or {})[extra])
+                except KeyError:
+                    extras_not_found.add(extra)
+            if extras_not_found:
+                warnings.warn(ExtrasError(sorted(extras_not_found)))
             return sorted(set(requires))
         else:
             return filter_requirements_with_extras(metadata.run_requires, extras)
@@ -254,9 +260,11 @@ class Candidate:
             result.update(
                 {
                     self.req.vcs: self.req.repo,
-                    "ref": self.req.ref if self.req.editable else self.revision,
+                    "ref": self.req.ref,
                 }
             )
+            if not self.req.editable:
+                result.update(revision=self.revision)
         elif not self.req.is_named:
             if self.req.is_file_or_url and self.req.is_local_dir:
                 result.update(path=path_replace(project_root, ".", self.req.str_path))
@@ -271,6 +279,6 @@ class Candidate:
     def format(self) -> str:
         """Format for output."""
         return (
-            f"{stream.green(self.name, bold=True)} "
-            f"{stream.yellow(str(self.version))}"
+            f"{termui.green(self.name, bold=True)} "
+            f"{termui.yellow(str(self.version))}"
         )
