@@ -30,6 +30,7 @@ from pip._internal.req.constructors import (
 )
 from pip._internal.req.req_file import parse_requirements
 from pip._internal.utils import logging as pip_logging
+from pip._internal.utils.filesystem import directory_size, file_size, find_files
 from pip._internal.utils.hashes import FAVORITE_HASH, STRONG_HASHES
 from pip._internal.utils.temp_dir import global_tempdir_manager
 from pip._internal.utils.urls import path_to_url, url_to_path
@@ -49,8 +50,7 @@ class InstallCommand(_InstallCommand):
         super().__init__(name="InstallCommand", summary="Install packages.")
 
 
-def get_abi_tag(python_version):
-    # type: (Tuple[int, int]) -> Optional[str]
+def get_abi_tag(python_version: Tuple[int, int]) -> Optional[str]:
     """Return the ABI tag based on SOABI (if available) or emulate SOABI
     (CPython 2, PyPy).
     A replacement for pip._internal.models.pep425tags:get_abi_tag()
@@ -70,7 +70,7 @@ def get_abi_tag(python_version):
 
     soabi = get_config_var("SOABI")
     impl = get_abbr_impl()
-    abi = None  # type: Optional[str]
+    abi: Optional[str] = None
 
     if not soabi and impl in {"cp", "pp"} and hasattr(sys, "maxunicode"):
         d = ""
