@@ -4,7 +4,7 @@ from pathlib import Path
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 
-class SetupReader(object):
+class SetupReader:
     """
     Class that reads a setup.py file without executing it.
     """
@@ -21,8 +21,8 @@ class SetupReader(object):
 
     @classmethod
     def read_from_directory(
-        cls, directory
-    ):  # type: (Union[str, Path]) -> Dict[str, Union[List, Dict]]
+        cls, directory: Union[str, Path]
+    ) -> Dict[str, Union[List, Dict]]:
         if isinstance(directory, str):
             directory = Path(directory)
 
@@ -43,7 +43,7 @@ class SetupReader(object):
         return result
 
     @classmethod
-    def _is_empty_result(cls, result):  # type: (Dict[str, Any]) -> bool
+    def _is_empty_result(cls, result: Dict[str, Any]) -> bool:
         return (
             not result["install_requires"]
             and not result["extras_require"]
@@ -51,10 +51,10 @@ class SetupReader(object):
         )
 
     def read_pyproject_toml(self, filepath):
-        from pdm.pep517.metadata import Metadata
+        from pdm.project.metadata import MutableMetadata
 
         try:
-            metadata = Metadata(filepath)
+            metadata = MutableMetadata(filepath)
         except ValueError:
             return {}
         return {
@@ -65,9 +65,7 @@ class SetupReader(object):
             "python_requires": metadata.requires_python,
         }
 
-    def read_setup_py(
-        self, filepath
-    ):  # type: (Union[str, Path]) -> Dict[str, Union[List, Dict]]
+    def read_setup_py(self, filepath: Union[str, Path]) -> Dict[str, Union[List, Dict]]:
 
         if isinstance(filepath, str):
             filepath = Path(filepath)
@@ -95,8 +93,8 @@ class SetupReader(object):
         return result
 
     def read_setup_cfg(
-        self, filepath
-    ):  # type: (Union[str, Path]) -> Dict[str, Union[List, Dict]]
+        self, filepath: Union[str, Path]
+    ) -> Dict[str, Union[List, Dict]]:
         parser = ConfigParser()
 
         parser.read(str(filepath))
@@ -144,8 +142,8 @@ class SetupReader(object):
         }
 
     def _find_setup_call(
-        self, elements
-    ):  # type: (List[Any]) -> Tuple[Optional[ast.Call], Optional[List[Any]]]
+        self, elements: List[Any]
+    ) -> Tuple[Optional[ast.Call], Optional[List[Any]]]:
         funcdefs = []
         for i, element in enumerate(elements):
             if isinstance(element, ast.If) and i == len(elements) - 1:
@@ -192,8 +190,8 @@ class SetupReader(object):
         return self._find_sub_setup_call(funcdefs)
 
     def _find_sub_setup_call(
-        self, elements
-    ):  # type: (List[Any]) -> Tuple[Optional[ast.Call], Optional[List[Any]]]
+        self, elements: List[Any]
+    ) -> Tuple[Optional[ast.Call], Optional[List[Any]]]:
         for element in elements:
             if not isinstance(element, (ast.FunctionDef, ast.If)):
                 continue
@@ -208,9 +206,7 @@ class SetupReader(object):
 
         return None, None
 
-    def _find_install_requires(
-        self, call, body
-    ):  # type: (ast.Call, Iterable[Any]) -> List[str]
+    def _find_install_requires(self, call: ast.Call, body: Iterable[Any]) -> List[str]:
         install_requires = []
         value = self._find_in_call(call, "install_requires")
         if value is None:
@@ -251,8 +247,8 @@ class SetupReader(object):
         return install_requires
 
     def _find_extras_require(
-        self, call, body
-    ):  # type: (ast.Call, Iterable[Any]) -> Dict[str, List]
+        self, call: ast.Call, body: Iterable[Any]
+    ) -> Dict[str, List]:
         extras_require = {}
         value = self._find_in_call(call, "extras_require")
         if value is None:
@@ -303,8 +299,8 @@ class SetupReader(object):
         return extras_require
 
     def _find_single_string(
-        self, call, body, name
-    ):  # type: (ast.Call, List[Any], str) -> Optional[str]
+        self, call: ast.Call, body: List[Any], name: str
+    ) -> Optional[str]:
         value = self._find_in_call(call, name)
         if value is None:
             # Trying to find in kwargs
@@ -339,12 +335,12 @@ class SetupReader(object):
             if variable is not None and isinstance(variable, ast.Str):
                 return variable.s
 
-    def _find_in_call(self, call, name):  # type: (ast.Call, str) -> Optional[Any]
+    def _find_in_call(self, call: ast.Call, name: str) -> Optional[Any]:
         for keyword in call.keywords:
             if keyword.arg == name:
                 return keyword.value
 
-    def _find_call_kwargs(self, call):  # type: (ast.Call) -> Optional[Any]
+    def _find_call_kwargs(self, call: ast.Call) -> Optional[Any]:
         kwargs = None
         for keyword in call.keywords:
             if keyword.arg is None:
@@ -352,9 +348,7 @@ class SetupReader(object):
 
         return kwargs
 
-    def _find_variable_in_body(
-        self, body, name
-    ):  # type: (Iterable[Any], str) -> Optional[Any]
+    def _find_variable_in_body(self, body: Iterable[Any], name: str) -> Optional[Any]:
         found = None
         for elem in body:
             if found:
@@ -370,7 +364,7 @@ class SetupReader(object):
                 if target.id == name:
                     return elem.value
 
-    def _find_in_dict(self, dict_, name):  # type: (ast.Call, str) -> Optional[Any]
+    def _find_in_dict(self, dict_: ast.Call, name: str) -> Optional[Any]:
         for key, val in zip(dict_.keys, dict_.values):
             if isinstance(key, ast.Str) and key.s == name:
                 return val
