@@ -1,10 +1,10 @@
-import re
 from functools import lru_cache
-from typing import Iterable, List, Optional, Set, Tuple, Union
+from typing import Iterable, List, Set, Tuple, Union, cast
 
 from pip._vendor.packaging.specifiers import SpecifierSet
 
 from pdm.exceptions import InvalidPyVersion
+from pdm.models.versions import Version
 
 
 @lru_cache()
@@ -16,113 +16,79 @@ def get_specifier(version_str: Union[SpecifierSet, str]) -> SpecifierSet:
     return SpecifierSet(version_str)
 
 
-def parse_version_tuple(version: str) -> Tuple[Union[int, str], ...]:
-    version = re.sub(r"(?<!\.)\*", ".*", version)
-    try:
-        return tuple(int(v) if v != "*" else v for v in version.split("."))[:3]
-    except ValueError:
-        raise InvalidPyVersion(
-            f"{version}: Prereleases or postreleases are not supported "
-            "for python version specifers."
-        )
+def _normalize_op_specifier(op: str, version_str: str) -> Tuple[str, Version]:
+    version = Version(version_str)
+    if version.is_wildcard:
+        if op == "==":
+            op = "~="
+            version[-1] = 0
+        elif op == ">":  # >X.Y.* => >=X.Y+1.0
+            op = ">="
+            version = version.bump(-2)
+        elif op in ("<", ">=", "<="):
+            # <X.Y.* => <X.Y.0
+            # >=X.Y.* => >=X.Y.0
+            # <=X.Y.* => <X.Y.0
+            version[-1] = 0
+            if op == "<=":
+                op = "<"
+        elif op != "!=":
+            raise InvalidPyVersion(f"Unsupported version specifier: {op}{version}")
 
-
-def bump_version(
-    version: Tuple[Union[int, str], ...], index: int = -1
-) -> Tuple[int, ...]:
-    head, value = version[:index], int(version[index])
-    new_version = _complete_version((*head, value + 1))
-    return new_version  # type: ignore
-
-
-def _complete_version(
-    version: Tuple[int, ...], complete_with: int = 0
-) -> Tuple[int, ...]:
-    assert len(version) <= 3, version
-    return version + (3 - len(version)) * (complete_with,)
-
-
-def _version_part_match(
-    valid_version: Tuple[int, ...], target_version: Tuple[int, ...]
-) -> bool:
-    return target_version[: len(valid_version)] == valid_version
-
-
-def _convert_to_version(version_tuple: Tuple[Union[int, str], ...]) -> str:
-    return ".".join(str(i) for i in version_tuple)
+    if op != "~=" and not (op == "!=" and version.is_wildcard):
+        # Don't complete with .0 for ~=3.5 and !=3.4.*:
+        version = version.complete()
+    return op, version
 
 
 class PySpecSet(SpecifierSet):
+    """A custom SpecifierSet that supports merging with logic operators (&, |)."""
+
     # TODO: fetch from python.org and cache
-    MAX_PY_VERSIONS = {
-        (2,): 7,
-        (2, 0): 1,
-        (2, 1): 3,
-        (2, 2): 3,
-        (2, 3): 7,
-        (2, 4): 6,
-        (2, 5): 6,
-        (2, 6): 9,
-        (2, 7): 18,
-        (3, 0): 1,
-        (3, 1): 5,
-        (3, 2): 6,
-        (3, 3): 7,
-        (3, 4): 10,
-        (3, 5): 10,
-        (3, 6): 12,
-        (3, 7): 9,
-        (3, 8): 6,
+    PY_MAX_MINOR_VERSION = {
+        Version("2"): 7,
+        Version("2.0"): 1,
+        Version("2.1"): 3,
+        Version("2.2"): 3,
+        Version("2.3"): 7,
+        Version("2.4"): 6,
+        Version("2.5"): 6,
+        Version("2.6"): 9,
+        Version("2.7"): 18,
+        Version("3.0"): 1,
+        Version("3.1"): 5,
+        Version("3.2"): 6,
+        Version("3.3"): 7,
+        Version("3.4"): 10,
+        Version("3.5"): 10,
+        Version("3.6"): 13,
+        Version("3.7"): 10,
+        Version("3.8"): 8,
+        Version("3.9"): 2,
     }
-    MAX_MAJOR_VERSION = 4
-    MIN_VERSION = (-1, -1, -1)
-    MAX_VERSION = (99, 99, 99)
+    MAX_MAJOR_VERSION = Version("4")
 
     def __init__(self, version_str: str = "", analyze: bool = True) -> None:
         if version_str == "*":
             version_str = ""
         super().__init__(version_str)
-        self._lower_bound = self.MIN_VERSION  # type: Tuple[int, int, int]
-        self._upper_bound = self.MAX_VERSION  # type: Tuple[int, int, int]
-        self._excludes = []  # type: List[Tuple[Union[int, str], ...]]
+        self._lower_bound = Version.MIN
+        self._upper_bound = Version.MAX
+        self._excludes: List[Version] = []
         if version_str and analyze:
             self._analyze_specifiers()
 
     def _analyze_specifiers(self) -> None:
         # XXX: Prerelease or postrelease specifiers will fail here, but I guess we can
         # just ignore them for now.
-        lower_bound, upper_bound = self.MIN_VERSION, self.MAX_VERSION
-        excludes = set()  # type: Set[Tuple[Union[int, str], ...]]
+        lower_bound, upper_bound = Version.MIN, Version.MAX
+        excludes: Set[Version] = set()
         for spec in self:
-            op, version = spec.operator, spec.version
-            version = parse_version_tuple(version)
-            if version[-1] == "*":
-                if op == "==":
-                    op = "~="
-                    version = version[:-1] + (0,)
-                elif op == "!=":
-                    excludes.add(version)
-                    continue
-                elif op == ">":  # >X.Y.* => >=X.Y+1.0
-                    op = ">="
-                    version = bump_version(version, -2)
-                elif op in ("<", ">=", "<="):
-                    # <X.Y.* => <X.Y.0
-                    # >=X.Y.* => >=X.Y.0
-                    # <=X.Y.* => <X.Y.0
-                    version = version[:-1] + (0,)
-                    if op == "<=":
-                        op = "<"
-                else:
-                    raise InvalidPyVersion(
-                        f"Unsupported version specifier: {spec.op}{spec.version}"
-                    )
+            op, version = _normalize_op_specifier(spec.operator, spec.version)
 
-            if op != "~=":
-                version = _complete_version(version)
             if op in ("==", "==="):
                 lower_bound = version
-                upper_bound = bump_version(version)
+                upper_bound = version.bump()
                 break
             if op == "!=":
                 excludes.add(version)
@@ -130,74 +96,66 @@ class PySpecSet(SpecifierSet):
                 if op == ">=":
                     new_lower = version
                 else:
-                    new_lower = bump_version(version)
+                    new_lower = version.bump()
                 if new_lower > lower_bound:
                     lower_bound = new_lower
             elif op[0] == "<":
                 if op == "<=":
-                    new_upper = bump_version(version)
+                    new_upper = version.bump()
                 else:
                     new_upper = version
                 if new_upper < upper_bound:
                     upper_bound = new_upper
             elif op == "~=":
-                new_lower = _complete_version(version)
-                new_upper = bump_version(version, -2)
+                new_lower = version.complete()
+                new_upper = version.bump(-2)
                 if new_upper < upper_bound:
                     upper_bound = new_upper
                 if new_lower > lower_bound:
                     lower_bound = new_lower
             else:
-                raise InvalidPyVersion(
-                    f"Unsupported version specifier: {spec.op}{spec.version}"
-                )
-        self._reorganize(lower_bound, upper_bound, excludes)
+                raise InvalidPyVersion(f"Unsupported version specifier: {op}{version}")
+        self._rearrange(lower_bound, upper_bound, excludes)
 
     @classmethod
     def _merge_bounds_and_excludes(
         cls,
-        lower: Tuple[int, int, int],
-        upper: Tuple[int, int, int],
-        excludes: Iterable[Tuple[Union[int, str], ...]],
-    ) -> Tuple[
-        Tuple[int, int, int], Tuple[int, int, int], List[Tuple[Union[int, str], ...]]
-    ]:
-        def comp_key(item):
-            # .* constraints are always considered before concrete constraints.
-            return tuple(e if isinstance(e, int) else -1 for e in item)
-
-        sorted_excludes = sorted(excludes, key=comp_key)
+        lower: Version,
+        upper: Version,
+        excludes: Iterable[Version],
+    ) -> Tuple[Version, Version, List[Version]]:
+        sorted_excludes = sorted(excludes)
         wildcard_excludes = set(
-            version[:-1] for version in sorted_excludes if version[-1] == "*"
+            version[:-1] for version in sorted_excludes if version.is_wildcard
         )
         # Remove versions that are already excluded by another wildcard exclude.
         sorted_excludes = [
             version
             for version in sorted_excludes
-            if version[-1] == "*"
-            or not any(_version_part_match(wv, version) for wv in wildcard_excludes)
+            if version.is_wildcard
+            or not any(version.startswith(wv) for wv in wildcard_excludes)
         ]
 
-        if lower == cls.MIN_VERSION and upper == cls.MAX_VERSION:
+        if lower == Version.MIN and upper == Version.MAX:
             # Nothing we can do here, it is a non-constraint.
             return lower, upper, sorted_excludes
 
         for version in list(sorted_excludes):  # from to low to high
-            if comp_key(version) >= comp_key(upper):
+            if version >= upper:
                 sorted_excludes[:] = []
                 break
 
-            if version[-1] == "*":
-                valid_length = len(version) - 1
+            if version.is_wildcard:
+                valid_length = len(version._version) - 1
                 valid_version = version[:valid_length]
 
                 if valid_version < lower[:valid_length]:
                     # Useless excludes
                     sorted_excludes.remove(version)
-                elif _version_part_match(valid_version, lower):
+                elif lower.startswith(valid_version):
                     # The lower bound is excluded, e.g: >=3.7.3,!=3.7.*
                     # bump the lower version in the last common bit: >=3.8.0
-                    lower = bump_version(version, -2)
+                    lower = version.bump(-2)
                     sorted_excludes.remove(version)
                 else:
                     break
@@ -205,28 +163,26 @@ class PySpecSet(SpecifierSet):
                 if version < lower:
                     sorted_excludes.remove(version)
                 elif version == lower:
-                    lower = bump_version(version)
+                    lower = version.bump()
                     sorted_excludes.remove(version)
                 else:
                     break
-        for version in sorted_excludes[::-1]:  # from high to low
-            if comp_key(version) >= comp_key(upper):
+        for version in reversed(sorted_excludes):  # from high to low
+            if version >= upper:
                 sorted_excludes.remove(version)
                 continue
 
-            if version[-1] == "*":
-                valid_length = len(version) - 1
+            if version.is_wildcard:
+                valid_length = len(version._version) - 1
                 valid_version = version[:valid_length]
 
-                if _version_part_match(valid_version, upper):
-                    # The upper bound is excluded, e.g: <3.7.3,!=3.7.*
+                if upper.startswith(valid_version) or version.bump(-2) == upper:
+                    # Case 1: The upper bound is excluded, e.g: <3.7.3,!=3.7.*
                     # set the upper to the zero version: <3.7.0
-                    upper = _complete_version(valid_version)
-                    sorted_excludes.remove(version)
-                elif bump_version(version, -2) == upper:
-                    # The upper bound is adjacent to the excluded one: <3.7.0,!=3.6.*
+                    # Case 2: The upper bound is adjacent to the excluded one,
+                    # e.g: <3.7.0,!=3.6.*
                     # Move the upper bound to below the excluded: <3.6.0
-                    upper = valid_version + (0,)
+                    upper = valid_version.complete()
                     sorted_excludes.remove(version)
                 else:
                     break
@@ -235,7 +191,10 @@ class PySpecSet(SpecifierSet):
 
         return lower, upper, sorted_excludes
 
-    def _reorganize(self, lower_bound, upper_bound, excludes):
+    def _rearrange(
+        self, lower_bound: Version, upper_bound: Version, excludes: Iterable[Version]
+    ) -> None:
+        """Rearrange the version bounds with the given inputs."""
         (
             self._lower_bound,
             self._upper_bound,
@@ -257,20 +216,19 @@ class PySpecSet(SpecifierSet):
 
     @property
     def is_impossible(self) -> bool:
-        if (
-            self._lower_bound is self.MIN_VERSION
-            or self._upper_bound is self.MAX_VERSION
-        ):
+        """Check whether the specifierset contains any valid versions."""
+        if self._lower_bound == Version.MIN or self._upper_bound == Version.MAX:
             return False
         return self._lower_bound >= self._upper_bound
 
     @property
     def is_allow_all(self) -> bool:
+        """Return True if the specifierset accepts all versions."""
         if self.is_impossible:
             return False
         return (
-            self._lower_bound == self.MIN_VERSION
-            and self._upper_bound == self.MAX_VERSION
+            self._lower_bound == Version.MIN
+            and self._upper_bound == Version.MAX
             and not self._excludes
         )
 
@@ -288,18 +246,17 @@ class PySpecSet(SpecifierSet):
             lower = lower[:-1]
         if upper[-1] == 0:
             upper = upper[:-1]
-        lower = "" if lower == self.MIN_VERSION else f">={_convert_to_version(lower)}"
-        upper = "" if upper == self.MAX_VERSION else f"<{_convert_to_version(upper)}"
-        excludes = ",".join(
-            f"!={_convert_to_version(version)}" for version in self._excludes
-        )
+        lower_str = "" if lower == Version.MIN else f">={lower}"
+        upper_str = "" if upper == Version.MAX else f"<{upper}"
+        excludes_str = ",".join(f"!={version}" for version in self._excludes)
 
-        return ",".join(filter(None, [lower, upper, excludes]))
+        return ",".join(filter(None, [lower_str, upper_str, excludes_str]))
 
     def __repr__(self) -> str:
         return f"<PySpecSet {self}>"
 
     def copy(self) -> "PySpecSet":
+        """Create a new specifierset that is same as the original one."""
         if self.is_impossible:
             return ImpossiblePySpecSet()
         instance = self.__class__(str(self), False)
@@ -320,7 +277,7 @@ class PySpecSet(SpecifierSet):
         excludes = set(rv._excludes) | set(other._excludes)
         lower = max(rv._lower_bound, other._lower_bound)
         upper = min(rv._upper_bound, other._upper_bound)
-        rv._reorganize(lower, upper, excludes)
+        rv._rearrange(lower, upper, excludes)
         return rv
 
     @lru_cache()
@@ -342,48 +299,56 @@ class PySpecSet(SpecifierSet):
             excludes.update(
                 self._populate_version_range(left._upper_bound, right._lower_bound)
             )
-        rv._reorganize(lower, upper, excludes)
+        rv._rearrange(lower, upper, excludes)
         return rv
 
-    def _populate_version_range(self, lower, upper):
-        assert lower and upper and lower < upper
+    def _populate_version_range(
+        self, lower: Version, upper: Version
+    ) -> Iterable[Version]:
+        """Expand the version range to a collection of versions to exclude,
+        taking the released python versions into consideration.
+        """
+        assert lower < upper
         prev = lower
         while prev < upper:
-            if prev[-2:] == (0, 0):
-                cur = bump_version(prev, 0)
-                if cur <= upper:  # X.0.0 -> X+1.0.0
-                    yield (prev[0], "*")
+            if prev[-2:] == Version((0, 0)):  # X.0.0
+                cur = prev.bump(0)  # X+1.0.0
+                if cur <= upper:  # It is still within the range
+                    yield Version((prev[0], "*"))  # Exclude the whole major series: X.*
                     prev = cur
                     continue
-            if prev[-1] == 0:
-                cur = bump_version(prev, 1)
-                if cur <= upper:  # X.Y.0 -> X.Y+1.0
-                    yield (*prev[:2], "*")
+            if prev[-1] == 0:  # X.Y.0
+                cur = prev.bump(1)  # X.Y+1.0
+                if cur <= upper:  # It is still within the range
+                    yield prev[:2].complete("*")  # Exclude X.Y.*
                     prev = (
-                        bump_version(prev, 0)
-                        if cur[0] < 3 and cur[1] > self.MAX_PY_VERSIONS[(cur[0],)]
+                        prev.bump(0)
+                        if cur.is_py2
+                        and cast(int, cur[1]) > self.PY_MAX_MINOR_VERSION[cur[:1]]
                         else cur
-                    )
+                    )  # If prev is 2.7, next is 3.0, otherwise next is X.Y+1.0
                     continue
-                while prev < upper:  # X.Y.0 -> X.Y.Z
+                while prev < upper:
+                    # Iterate each version from X.Y.0(prev) to X.Y.Z(upper)
                     yield prev
-                    prev = bump_version(prev)
+                    prev = prev.bump()
                 break
-            # no wildcard is available
-            cur = bump_version(prev, 1)
-            if cur <= upper:  # X.Y.Z -> X.Y+1.0
-                current_max = self.MAX_PY_VERSIONS[(*prev[:2],)]
-                for z in range(prev[2], current_max + 1):
-                    yield (*prev[:2], z)
+            # Can't produce any wildcard versions
+            cur = prev.bump(1)
+            if cur <= upper:  # X.Y+1.0 is still within the range
+                current_max = self.PY_MAX_MINOR_VERSION[prev[:2]]
+                for z in range(cast(int, prev[2]), current_max + 1):
+                    yield prev[:2].complete(z)
                 prev = (
-                    bump_version(prev, 0)
-                    if cur[0] < 3 and cur[1] > self.MAX_PY_VERSIONS[(cur[0],)]
+                    prev.bump(0)
+                    if cur.is_py2
+                    and cast(int, cur[1]) > self.PY_MAX_MINOR_VERSION[cur[:1]]
                     else cur
                 )
-            else:  # X.Y.Z -> X.Y.W
+            else:  # Produce each version from X.Y.Z to X.Y.W
                 while prev < upper:
                     yield prev
-                    prev = bump_version(prev)
+                    prev = prev.bump()
                 break
 
     @lru_cache()
@@ -393,10 +358,10 @@ class PySpecSet(SpecifierSet):
         if self.is_allow_all:
             return True
         other = type(self)(str(other))
-        if other._upper_bound[0] >= self.MAX_MAJOR_VERSION:
+        if other._upper_bound >= self.MAX_MAJOR_VERSION:
             # XXX: narrow down the upper bound to ``MAX_MAJOR_VERSION``
             # So that `>=3.6,<4.0` is considered a superset of `>=3.7`, see issues/66
-            other._upper_bound = (self.MAX_MAJOR_VERSION, 0, 0)
+            other._upper_bound = self.MAX_MAJOR_VERSION.complete(0)
         lower, upper, excludes = self._merge_bounds_and_excludes(
             other._lower_bound, other._upper_bound, self._excludes
         )
@@ -416,8 +381,9 @@ class PySpecSet(SpecifierSet):
         if self.is_impossible:
             return False
         other = type(self)(str(other))
-        if other._upper_bound[0] >= self.MAX_MAJOR_VERSION:
-            other._upper_bound = self.MAX_VERSION
+        if other._upper_bound >= self.MAX_MAJOR_VERSION:
+            # Relax the upper bound to max version
+            other._upper_bound = Version.MAX
         if other.is_allow_all:
             return True
         lower, upper, excludes = self._merge_bounds_and_excludes(
@@ -468,24 +434,20 @@ class PySpecSet(SpecifierSet):
             )
         return " and ".join(result)
 
-    def max_major_minor_version(self) -> Optional[Tuple[int, int]]:
-        if self._upper_bound == self.MAX_VERSION:
-            return None
-        if self._upper_bound[-1] == 0:
-            return self._upper_bound[0], self._upper_bound[1] - 1
-        else:
-            return self._upper_bound[:2]
-
     def supports_py2(self) -> bool:
-        return self._lower_bound[0] < 3
+        return self._lower_bound.is_py2
 
 
 class ImpossiblePySpecSet(PySpecSet):
+    """
+    A special subclass of PySpecSet that references to an impossible specifier set.
+    """
+
     def __init__(self, version_str="", analyze=True):
         super().__init__(version_str=version_str, analyze=False)
         # Make sure the spec set is impossible
-        self._lower_bound = self.MAX_VERSION
-        self._upper_bound = self.MIN_VERSION
+        self._lower_bound = Version.MAX
+        self._upper_bound = Version.MIN
 
     @property
     def is_impossible(self):
