@@ -5,10 +5,11 @@ from pathlib import Path
 from typing import Any, Callable, Dict, Iterable, Optional, TypeVar
 
 import appdirs
+import click
 import tomlkit
 
+from pdm import termui
 from pdm.exceptions import NoConfigError
-from pdm.iostream import stream
 from pdm.utils import get_pypi_source
 
 T = TypeVar("T")
@@ -72,7 +73,7 @@ class Config(MutableMapping):
     HOME_CONFIG = Path.home() / ".pdm" / "config.toml"
 
     pypi_url, verify_ssl = get_pypi_source()
-    _config_map = {
+    _config_map: Dict[str, ConfigItem] = {
         "cache_dir": ConfigItem(
             "The root directory of cached files", appdirs.user_cache_dir("pdm"), True
         ),
@@ -125,7 +126,7 @@ class Config(MutableMapping):
             env_var="PDM_USE_VENV",
             coerce=ensure_boolean,
         ),
-    }  # type: Dict[str, ConfigItem]
+    }
     del pypi_url, verify_ssl
 
     @classmethod
@@ -185,8 +186,8 @@ class Config(MutableMapping):
         value = self._config_map[key].coerce(value)
         env_var = self._config_map[key].env_var
         if env_var is not None and env_var in os.environ:
-            stream.echo(
-                stream.yellow(
+            click.echo(
+                termui.yellow(
                     "WARNING: the config is shadowed by env var '{}', "
                     "the value set won't take effect.".format(env_var)
                 )
@@ -210,8 +211,8 @@ class Config(MutableMapping):
         else:
             env_var = self._config_map[key].env_var
             if env_var is not None and env_var in os.environ:
-                stream.echo(
-                    stream.yellow(
+                click.echo(
+                    termui.yellow(
                         "WARNING: the config is shadowed by env var '{}', "
                         "set value won't take effect.".format(env_var)
                     )
